@@ -98,33 +98,42 @@ void SciTEBase::SetLanguageMenu() {
 	}
 }
 
+// Null except on Windows where it may be overridden
+void SciTEBase::ReadEmbeddedProperties() {
+}
+
 const GUI::gui_char propLocalFileName[] = GUI_TEXT("SciTE.properties");
 const GUI::gui_char propDirectoryFileName[] = GUI_TEXT("SciTEDirectory.properties");
 
+void SciTEBase::ReadEnvironment() {
+#if defined(__unix__)
+	extern char **environ;
+	char **e = environ;
+#else
+	char **e = _environ;
+#endif
+	for (; e && *e; e++) {
+		char key[1024];
+		char *k = *e;
+		char *v = strchr(k, '=');
+		if (v && (static_cast<size_t>(v - k) < sizeof(key))) {
+			memcpy(key, k, v - k);
+			key[v - k] = '\0';
+			propsPlatform.Set(key, v + 1);
+		}
+	}
+}
+
 /**
 Read global and user properties files.
 */
 void SciTEBase::ReadGlobalPropFile() {
-#if defined(__unix__)
-	extern char **environ;
-	char **e=environ;
-#else
-	char **e=_environ;
-#endif
-	for (; e && *e; e++) {
-		char key[1024];
-		char *k=*e;
-		char *v=strchr(k,'=');
-		if (v && (static_cast<size_t>(v-k) < sizeof(key))) {
-			memcpy(key, k, v-k);
-			key[v-k] = '\0';
-			propsEmbed.Set(key, v+1);
-		}
-	}
-
 	std::string excludes;
 	std::string includes;
 
+	// Want to apply imports.exclude and imports.include but these may well be in
+	// user properties.
+
 	for (int attempt=0; attempt<2; attempt++) {
 
 		std::string excludesRead = props.GetString("imports.exclude");
@@ -139,6 +148,8 @@ void SciTEBase::ReadGlobalPropFile() {
 
 		importFiles.clear();
 
+		ReadEmbeddedProperties();
+
 		propsBase.Clear();
 		FilePath propfileBase = GetDefaultPropertiesFileName();
 		propsBase.Read(propfileBase, propfileBase.Directory(), filter, &importFiles, 0);
@@ -266,29 +277,29 @@ StyleDefinition SciTEBase::StyleDefinitionFor(int style) {
 
 void SciTEBase::SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd) {
 	if (sd.specified & StyleDefinition::sdItalics)
-		win.Send(SCI_STYLESETITALIC, style, sd.italics ? 1 : 0);
+		win.Call(SCI_STYLESETITALIC, style, sd.italics ? 1 : 0);
 	if (sd.specified & StyleDefinition::sdWeight)
-		win.Send(SCI_STYLESETWEIGHT, style, sd.weight);
+		win.Call(SCI_STYLESETWEIGHT, style, sd.weight);
 	if (sd.specified & StyleDefinition::sdFont)
-		win.SendPointer(SCI_STYLESETFONT, style,
-			const_cast<char *>(sd.font.c_str()));
+		win.CallString(SCI_STYLESETFONT, style,
+			sd.font.c_str());
 	if (sd.specified & StyleDefinition::sdFore)
-		win.Send(SCI_STYLESETFORE, style, sd.ForeAsLong());
+		win.Call(SCI_STYLESETFORE, style, sd.ForeAsLong());
 	if (sd.specified & StyleDefinition::sdBack)
-		win.Send(SCI_STYLESETBACK, style, sd.BackAsLong());
+		win.Call(SCI_STYLESETBACK, style, sd.BackAsLong());
 	if (sd.specified & StyleDefinition::sdSize)
-		win.Send(SCI_STYLESETSIZEFRACTIONAL, style, sd.FractionalSize());
+		win.Call(SCI_STYLESETSIZEFRACTIONAL, style, sd.FractionalSize());
 	if (sd.specified & StyleDefinition::sdEOLFilled)
-		win.Send(SCI_STYLESETEOLFILLED, style, sd.eolfilled ? 1 : 0);
+		win.Call(SCI_STYLESETEOLFILLED, style, sd.eolfilled ? 1 : 0);
 	if (sd.specified & StyleDefinition::sdUnderlined)
-		win.Send(SCI_STYLESETUNDERLINE, style, sd.underlined ? 1 : 0);
+		win.Call(SCI_STYLESETUNDERLINE, style, sd.underlined ? 1 : 0);
 	if (sd.specified & StyleDefinition::sdCaseForce)
-		win.Send(SCI_STYLESETCASE, style, sd.caseForce);
+		win.Call(SCI_STYLESETCASE, style, sd.caseForce);
 	if (sd.specified & StyleDefinition::sdVisible)
-		win.Send(SCI_STYLESETVISIBLE, style, sd.visible ? 1 : 0);
+		win.Call(SCI_STYLESETVISIBLE, style, sd.visible ? 1 : 0);
 	if (sd.specified & StyleDefinition::sdChangeable)
-		win.Send(SCI_STYLESETCHANGEABLE, style, sd.changeable ? 1 : 0);
-	win.Send(SCI_STYLESETCHARACTERSET, style, characterSet);
+		win.Call(SCI_STYLESETCHANGEABLE, style, sd.changeable ? 1 : 0);
+	win.Call(SCI_STYLESETCHARACTERSET, style, characterSet);
 }
 
 void SciTEBase::SetStyleBlock(GUI::ScintillaWindow &win, const char *lang, int start, int last) {
@@ -347,9 +358,9 @@ void SciTEBase::ForwardPropertyToEditor(const char *key) {
 	if (props.Exists(key)) {
 		std::string value = props.GetExpandedString(key);
 		wEditor.CallString(SCI_SETPROPERTY,
-						 reinterpret_cast<uptr_t>(key), value.c_str());
+						 UptrFromString(key), value.c_str());
 		wOutput.CallString(SCI_SETPROPERTY,
-						 reinterpret_cast<uptr_t>(key), value.c_str());
+						 UptrFromString(key), value.c_str());
 	}
 }
 
@@ -403,6 +414,8 @@ static const char *propertiesToForward[] = {
 //**\(\t"\*",\n\)
 	"asp.default.language",
 	"fold",
+	"fold.abl.comment.multiline",
+	"fold.abl.syntax.based",
 	"fold.asm.comment.explicit",
 	"fold.asm.comment.multiline",
 	"fold.asm.explicit.anywhere",
@@ -410,6 +423,8 @@ static const char *propertiesToForward[] = {
 	"fold.asm.explicit.start",
 	"fold.asm.syntax.based",
 	"fold.at.else",
+	"fold.baan.keywords.based",
+	"fold.baan.syntax.based",
 	"fold.basic.comment.explicit",
 	"fold.basic.explicit.anywhere",
 	"fold.basic.explicit.end",
@@ -425,6 +440,7 @@ static const char *propertiesToForward[] = {
 	"fold.cpp.explicit.anywhere",
 	"fold.cpp.explicit.end",
 	"fold.cpp.explicit.start",
+	"fold.cpp.preprocessor.at.else",
 	"fold.cpp.syntax.based",
 	"fold.d.comment.explicit",
 	"fold.d.comment.multiline",
@@ -456,6 +472,7 @@ static const char *propertiesToForward[] = {
 	"fold.verilog.flags",
 	"html.tags.case.sensitive",
 	"lexer.asm.comment.delimiter",
+	"lexer.baan.styling.within.preprocessor",
 	"lexer.caml.magic",
 	"lexer.cpp.allow.dollars",
 	"lexer.cpp.backquoted.strings",
@@ -469,6 +486,7 @@ static const char *propertiesToForward[] = {
 	"lexer.css.less.language",
 	"lexer.css.scss.language",
 	"lexer.d.fold.at.else",
+	"lexer.errorlist.escape.sequences",
 	"lexer.errorlist.value.separate",
 	"lexer.flagship.styling.within.preprocessor",
 	"lexer.haskell.allow.hash",
@@ -478,6 +496,8 @@ static const char *propertiesToForward[] = {
 	"lexer.haskell.import.safe",
 	"lexer.html.django",
 	"lexer.html.mako",
+	"lexer.json.allow.comments",
+	"lexer.json.escape.sequence",
 	"lexer.metapost.comment.process",
 	"lexer.metapost.interface.default",
 	"lexer.pascal.smart.highlighting",
@@ -634,7 +654,7 @@ void SciTEBase::ReadProperties() {
 				lexLPeg = wEditor.Call(SCI_GETLEXER);
 				const char *lexer = language.c_str() + language.find("_") + 1;
 				wEditor.CallReturnPointer(SCI_PRIVATELEXERCALL, SCI_SETLEXERLANGUAGE,
-					reinterpret_cast<sptr_t>(lexer));
+					SptrFromString(lexer));
 			}
 		} else {
 			wEditor.CallString(SCI_SETLEXERLANGUAGE, 0, language.c_str());
@@ -766,6 +786,7 @@ void SciTEBase::ReadProperties() {
 	CallChildren(SCI_SETMOUSESELECTIONRECTANGULARSWITCH, props.GetInt("selection.rectangular.switch.mouse", 0));
 	CallChildren(SCI_SETMULTIPLESELECTION, props.GetInt("selection.multiple", 1));
 	CallChildren(SCI_SETADDITIONALSELECTIONTYPING, props.GetInt("selection.additional.typing", 1));
+	CallChildren(SCI_SETMULTIPASTE, props.GetInt("selection.multipaste", 1));
 	CallChildren(SCI_SETADDITIONALCARETSBLINK, props.GetInt("caret.additional.blinks", 1));
 	CallChildren(SCI_SETVIRTUALSPACEOPTIONS, props.GetInt("virtual.space"));
 
@@ -790,7 +811,7 @@ void SciTEBase::ReadProperties() {
 		alphaIndicator = 30; //then set default value.
 	underIndicator = props.GetInt("indicators.under", 0) == 1;
 
-	closeFind = props.GetInt("find.close.on.find", 1);
+	closeFind = static_cast<CloseFind>(props.GetInt("find.close.on.find", 1));
 
 	const std::string controlCharSymbol = props.GetString("control.char.symbol");
 	if (controlCharSymbol.length()) {
@@ -918,6 +939,16 @@ void SciTEBase::ReadProperties() {
 	wEditor.CallString(SCI_AUTOCSETFILLUPS, 0,
 		autoCompleteFillUpCharacters.c_str());
 
+	sprintf(key, "autocomplete.%s.typesep", language.c_str());
+	autoCompleteTypeSeparator = props.GetExpandedString(key);
+	if (autoCompleteTypeSeparator == "")
+		autoCompleteTypeSeparator =
+			props.GetExpandedString("autocomplete.*.typesep");
+	if (autoCompleteTypeSeparator.length()) {
+		wEditor.Call(SCI_AUTOCSETTYPESEPARATOR,
+			static_cast<unsigned char>(autoCompleteTypeSeparator[0]));
+	}
+
 	sprintf(key, "autocomplete.%s.ignorecase", "*");
 	sval = props.GetNewExpandString(key);
 	autoCompleteIgnoreCase = sval == "1";
@@ -947,10 +978,11 @@ void SciTEBase::ReadProperties() {
 	jobQueue.timeCommands = props.GetInt("time.commands");
 
 	int blankMarginLeft = props.GetInt("blank.margin.left", 1);
+	int blankMarginLeftOutput = props.GetInt("output.blank.margin.left", blankMarginLeft);
 	int blankMarginRight = props.GetInt("blank.margin.right", 1);
 	wEditor.Call(SCI_SETMARGINLEFT, 0, blankMarginLeft);
 	wEditor.Call(SCI_SETMARGINRIGHT, 0, blankMarginRight);
-	wOutput.Call(SCI_SETMARGINLEFT, 0, blankMarginLeft);
+	wOutput.Call(SCI_SETMARGINLEFT, 0, blankMarginLeftOutput);
 	wOutput.Call(SCI_SETMARGINRIGHT, 0, blankMarginRight);
 
 	marginWidth = props.GetInt("margin.width");
@@ -966,7 +998,7 @@ void SciTEBase::ReadProperties() {
 
 	SetLineNumberWidth();
 
-	bufferedDraw = props.GetInt("buffered.draw", 1);
+	bufferedDraw = props.GetInt("buffered.draw");
 	wEditor.Call(SCI_SETBUFFEREDDRAW, bufferedDraw);
 	wOutput.Call(SCI_SETBUFFEREDDRAW, bufferedDraw);
 
@@ -1042,6 +1074,9 @@ void SciTEBase::ReadProperties() {
  	wEditor.Call(SCI_SETWRAPSTARTINDENT, props.GetInt("wrap.visual.startindent"));
  	wEditor.Call(SCI_SETWRAPINDENTMODE, props.GetInt("wrap.indent.mode"));
 
+	wEditor.Call(SCI_SETIDLESTYLING, props.GetInt("idle.styling", SC_IDLESTYLING_NONE));
+	wOutput.Call(SCI_SETIDLESTYLING, props.GetInt("output.idle.styling", SC_IDLESTYLING_NONE));
+
 	if (props.GetInt("os.x.home.end.keys")) {
 		AssignKey(SCK_HOME, 0, SCI_SCROLLTOSTART);
 		AssignKey(SCK_HOME, SCMOD_SHIFT, SCI_NULL);
@@ -1214,8 +1249,7 @@ void SciTEBase::ReadProperties() {
 		wEditor.Call(SCI_MARKERDEFINE, markerBookmark, SC_MARK_BOOKMARK);
 	} else {
 		// No bookmark.fore setting so display default pixmap.
-		wEditor.CallString(SCI_MARKERDEFINEPIXMAP, markerBookmark,
-			reinterpret_cast<char *>(bookmarkBluegem));
+		wEditor.CallPointer(SCI_MARKERDEFINEPIXMAP, markerBookmark, bookmarkBluegem);
 	}
 
 	wEditor.Call(SCI_SETSCROLLWIDTH, props.GetInt("horizontal.scroll.width", 2000));
@@ -1301,7 +1335,8 @@ void SciTEBase::ReadFontProperties() {
 		char propStr[256];
 		for (int i = 0; i < STYLE_MAX; i++) {
 			sprintf(key, "style.lpeg.%0d", i);
-			wEditor.Send(SCI_PRIVATELEXERCALL, i - STYLE_MAX, reinterpret_cast<sptr_t>(propStr));
+			wEditor.CallReturnPointer(SCI_PRIVATELEXERCALL, i - STYLE_MAX,
+				SptrFromString(propStr));
 			props.Set(key, static_cast<const char *>(propStr));
 		}
 		languageName = "lpeg";
@@ -1494,7 +1529,7 @@ void SciTEBase::ReadPropertiesInitial() {
 	wEditor.Call(SCI_SETWRAPMODE, wrap ? wrapStyle : SC_WRAP_NONE);
 	wOutput.Call(SCI_SETWRAPMODE, wrapOutput ? wrapStyle : SC_WRAP_NONE);
 
-	std::string menuLanguageProp = props.GetNewExpandString("menu.language");
+	std::string menuLanguageProp = props.GetExpandedString("menu.language");
 	std::replace(menuLanguageProp.begin(), menuLanguageProp.end(), '|', '\0');
 	const char *sMenuLanguage = menuLanguageProp.c_str();
 	while (*sMenuLanguage) {
