@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# Requires Python 2.7 or later
 
 from __future__ import with_statement
 from __future__ import unicode_literals
@@ -181,7 +182,11 @@ class XiteWin():
 	def OnCreate(self, hwnd):
 		self.win = hwnd
 		# Side effect: loads the DLL
-		x = ctypes.windll.SciLexer.Scintilla_DirectFunction
+		try:
+			x = ctypes.windll.SciLexer.Scintilla_DirectFunction
+		except OSError:
+			print("Can't find SciLexer.DLL")
+			sys.exit()
 		self.sciHwnd = user32.CreateWindowExW(0,
 			"Scintilla", "Source",
 			WS_CHILD | WS_VSCROLL | WS_HSCROLL | WS_CLIPCHILDREN,
@@ -298,16 +303,16 @@ class XiteWin():
 
 	def Open(self):
 		ofx = OPENFILENAME(self.win, "Open File")
-		opath = "\0" * 1024
-		ofx.lpstrFile = opath
+		opath = ctypes.create_unicode_buffer(1024)
+		ofx.lpstrFile = ctypes.addressof(opath)
 		filters = ["Python (.py;.pyw)|*.py;*.pyw|All|*.*"]
 		filterText = "\0".join([f.replace("|", "\0") for f in filters])+"\0\0"
 		ofx.lpstrFilter = filterText
 		if ctypes.windll.comdlg32.GetOpenFileNameW(ctypes.byref(ofx)):
-			absPath = opath.replace("\0", "")
+			absPath = opath.value.replace("\0", "")
 			self.GrabFile(absPath)
 			self.FocusOnEditor()
-			self.ed.LexerLanguage = "python"
+			self.ed.LexerLanguage = b"python"
 			self.ed.Lexer = self.ed.SCLEX_PYTHON
 			self.ed.SetKeyWords(0, b"class def else for from if import print return while")
 			for style in [k for k in self.ed.k if k.startswith("SCE_P_")]:
