@@ -116,14 +116,14 @@ void SciTEWin::WarnUser(int warnID) {
 
 	int flashLen = atoi(flashDuration);
 	if (flashLen) {
-		FlashThisWindow(reinterpret_cast<HWND>(wEditor.GetID()), flashLen);
+		FlashThisWindow(HwndOf(wEditor), flashLen);
 	}
 	PlayThisSound(sound, atoi(soundDuration), hMM);
 }
 
 bool SciTEWin::DialogHandled(GUI::WindowID id, MSG *pmsg) {
 	if (id) {
-		if (	::IsDialogMessageW(reinterpret_cast<HWND>(id), pmsg))
+		if (::IsDialogMessageW(static_cast<HWND>(id), pmsg))
 			return true;
 	}
 	return false;
@@ -170,19 +170,28 @@ bool SciTEWin::ModelessHandler(MSG *pmsg) {
 }
 
 //  DoDialog is a bit like something in PC Magazine May 28, 1991, page 357
-int SciTEWin::DoDialog(HINSTANCE hInst, const TCHAR *resName, HWND hWnd, DLGPROC lpProc) {
+int SciTEWin::DoDialog(const TCHAR *resName, DLGPROC lpProc) {
 	int result = static_cast<int>(
-		::DialogBoxParam(hInst, resName, hWnd, lpProc, reinterpret_cast<LPARAM>(this)));
+		::DialogBoxParam(hInstance, resName, MainHWND(), lpProc, reinterpret_cast<LPARAM>(this)));
 
 	if (result == -1) {
 		GUI::gui_string errorNum = GUI::StringFromInteger(::GetLastError());
 		GUI::gui_string msg = LocaliseMessage("Failed to create dialog box: ^0.", errorNum.c_str());
-		::MessageBoxW(hWnd, msg.c_str(), appName, MB_OK | MB_SETFOREGROUND);
+		::MessageBoxW(MainHWND(), msg.c_str(), appName, MB_OK | MB_SETFOREGROUND);
 	}
 
+	WindowSetFocus(wEditor);
+
 	return result;
 }
 
+HWND SciTEWin::CreateParameterisedDialog(LPCWSTR lpTemplateName, DLGPROC lpProc) {
+	return ::CreateDialogParamW(hInstance,
+		lpTemplateName,
+		MainHWND(),
+		lpProc,
+		reinterpret_cast<LPARAM>(this));
+}
 
 GUI::gui_string SciTEWin::DialogFilterFromProperty(const GUI::gui_char *filterProperty) {
 	GUI::gui_string filterText = filterProperty;
@@ -234,8 +243,7 @@ bool SciTEWin::OpenDialog(FilePath directory, const GUI::gui_char *filesFilter)
 	GUI::gui_char openName[maxBufferSize]; // maximum common dialog buffer size (says mfc..)
 	openName[0] = '\0';
 
-	OPENFILENAMEW ofn;
-	memset(&ofn, 0, sizeof(ofn));
+	OPENFILENAMEW ofn = {};
 	ofn.lStructSize = sizeof(ofn);
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
@@ -289,8 +297,7 @@ FilePath SciTEWin::ChooseSaveName(FilePath directory, const char *title, const G
 		if (!savePath.IsUntitled()) {
 			StringCopy(saveName, savePath.AsInternal());
 		}
-		OPENFILENAMEW ofn;
-		memset(&ofn, 0, sizeof(ofn));
+		OPENFILENAMEW ofn = {};
 		ofn.lStructSize = sizeof(ofn);
 		ofn.hwndOwner = MainHWND();
 		ofn.hInstance = hInstance;
@@ -372,8 +379,7 @@ void SciTEWin::SaveAsXML() {
 
 void SciTEWin::LoadSessionDialog() {
 	GUI::gui_char openName[MAX_PATH] = GUI_TEXT("");
-	OPENFILENAMEW ofn;
-	memset(&ofn, 0, sizeof(ofn));
+	OPENFILENAMEW ofn = {};
 	ofn.lStructSize = sizeof(ofn);
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
@@ -394,8 +400,7 @@ void SciTEWin::LoadSessionDialog() {
 void SciTEWin::SaveSessionDialog() {
 	GUI::gui_char saveName[MAX_PATH] = GUI_TEXT("\0");
 	StringCopy(saveName, GUI_TEXT("SciTE.session"));
-	OPENFILENAMEW ofn;
-	memset(&ofn, 0, sizeof(ofn));
+	OPENFILENAMEW ofn = {};
 	ofn.lStructSize = sizeof(ofn);
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
@@ -447,8 +452,8 @@ void SciTEWin::Print(
 
 	// See if a range has been selected
 	Sci_CharacterRange crange = GetSelection();
-	int startPos = crange.cpMin;
-	int endPos = crange.cpMax;
+	int startPos = static_cast<int>(crange.cpMin);
+	int endPos = static_cast<int>(crange.cpMax);
 
 	if (startPos == endPos) {
 		pdlg.Flags |= PD_NOSELECTION;
@@ -682,9 +687,9 @@ void SciTEWin::Print(
 		frPrint.chrg.cpMin = lengthPrinted;
 		frPrint.chrg.cpMax = lengthDoc;
 
-		lengthPrinted = wEditor.Call(SCI_FORMATRANGE,
+		lengthPrinted = wEditor.CallPointer(SCI_FORMATRANGE,
 		                           printPage,
-		                           reinterpret_cast<LPARAM>(&frPrint));
+		                           &frPrint);
 
 		if (printPage) {
 			if (footerFormat.size()) {
@@ -813,8 +818,7 @@ public:
 		::SendMessage(combo, CB_RESETCONTENT, 0, 0);
 		for (int i = 0; i < mem.Length(); i++) {
 			GUI::gui_string gs = GUI::StringFromUTF8(mem.At(i));
-			::SendMessageW(combo, CB_ADDSTRING, 0,
-				       reinterpret_cast<LPARAM>(gs.c_str()));
+			ComboBox_AddString(combo, gs.c_str());
 		}
 		if (useTop) {
 			::SendMessage(combo, CB_SETCURSEL, 0, 0);
@@ -828,10 +832,10 @@ static void FillComboFromProps(HWND combo, PropSetFile &props) {
 	const char *val;
 	if (props.GetFirst(key, val)) {
 		GUI::gui_string wkey = GUI::StringFromUTF8(key);
-		::SendMessage(combo, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(wkey.c_str()));
+		ComboBox_AddString(combo, wkey.c_str());
 		while (props.GetNext(key, val)) {
 			wkey = GUI::StringFromUTF8(key);
-			::SendMessage(combo, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(wkey.c_str()));
+			ComboBox_AddString(combo, wkey.c_str());
 		}
 	}
 }
@@ -955,14 +959,15 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		} else if ( (ControlIDOfWParam(wParam) == IDOK) ||
 		            (ControlIDOfWParam(wParam) == IDMARKALL) ) {
 			dlg.GrabFields();
-			if (closeFind) {
+			if (closeFind != Searcher::CloseFind::closePrevent) {
 				::EndDialog(hDlg, IDOK);
 				wFindReplace.Destroy();
 			}
 			if (ControlIDOfWParam(wParam) == IDMARKALL){
 				MarkAll(markWithBookMarks);
 			}
-			FindNext(reverseFind ^ IsKeyDown(VK_SHIFT));
+			// Holding the Shift key inverts the current reverse flag
+			FindNext(reverseFind != IsKeyDown(VK_SHIFT));
 			return TRUE;
 		} else if (ControlIDOfWParam(wParam) == IDFINDINSTYLE) {
 			if (FindReplaceAdvanced()) {
@@ -976,14 +981,14 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::FindDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::FindDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->FindMessage(hDlg, message, wParam);
 }
 
 BOOL SciTEWin::HandleReplaceCommand(int cmd, bool reverseDirection) {
 	if (!wFindReplace.GetID())
 		return TRUE;
-	HWND hwndFR = reinterpret_cast<HWND>(wFindReplace.GetID());
+	HWND hwndFR = HwndOf(wFindReplace);
 	DialogFindReplace dlg(hwndFR, FindReplaceAdvanced());
 	dlg.SetSearcher(this);
 
@@ -1054,7 +1059,7 @@ BOOL SciTEWin::ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::ReplaceDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::ReplaceDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->ReplaceMessage(hDlg, message, wParam);
 }
 
@@ -1093,7 +1098,7 @@ void SciTEWin::Find() {
 	if (wFindReplace.Created()) {
 		if (!replacing) {
 			SelectionIntoFind();
-			HWND hDlg = reinterpret_cast<HWND>(wFindReplace.GetID());
+			HWND hDlg = HwndOf(wFindReplace);
 			Dialog dlg(hDlg);
 			dlg.SetItemTextU(IDFINDWHAT, findWhat);
 			::SetFocus(hDlg);
@@ -1117,13 +1122,8 @@ void SciTEWin::Find() {
 
 		replacing = false;
 
-		int dialog_id = FindReplaceAdvanced() ? IDD_FIND_ADV : IDD_FIND;
-
-		wFindReplace = ::CreateDialogParamW(hInstance,
-											(LPCWSTR)MAKEINTRESOURCE(dialog_id),
-											MainHWND(),
-											reinterpret_cast<DLGPROC>(FindDlg),
-											reinterpret_cast<LPARAM>(this));
+		const int dialogID = FindReplaceAdvanced() ? IDD_FIND_ADV : IDD_FIND;
+		wFindReplace = CreateParameterisedDialog(MAKEINTRESOURCE(dialogID), FindDlg);
 		wFindReplace.Show();
 	}
 }
@@ -1260,8 +1260,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			if (::SHGetMalloc(&pShellMalloc) == NO_ERROR) {
 				// If we were able to get the shell malloc object,
 				// then proceed by initializing the BROWSEINFO stuct
-				BROWSEINFO info;
-				memset(&info, 0, sizeof(info));
+				BROWSEINFO info = {};
 				info.hwndOwner = hDlg;
 				info.pidlRoot = NULL;
 				TCHAR szDisplayName[MAX_PATH] = TEXT("");
@@ -1299,14 +1298,14 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::GrepDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::GrepDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->GrepMessage(hDlg, message, wParam);
 }
 
 void SciTEWin::FindInFiles() {
 	SelectionIntoFind();
 	if (wFindInFiles.Created()) {
-		HWND hDlg = reinterpret_cast<HWND>(wFindInFiles.GetID());
+		HWND hDlg = HwndOf(wFindInFiles);
 		Dialog dlg(hDlg);
 		dlg.SetItemTextU(IDFINDWHAT, findWhat);
 		::SetFocus(hDlg);
@@ -1321,9 +1320,7 @@ void SciTEWin::FindInFiles() {
 		FilePath findInDir = filePath.Directory();
 		props.Set("find.directory", findInDir.AsUTF8().c_str());
 	}
-
-	wFindInFiles = ::CreateDialogParam(hInstance, TEXT("Grep"), MainHWND(),
-		reinterpret_cast<DLGPROC>(GrepDlg), reinterpret_cast<sptr_t>(this));
+	wFindInFiles = CreateParameterisedDialog(TEXT("Grep"), GrepDlg);
 	wFindInFiles.Show();
 }
 
@@ -1331,7 +1328,7 @@ void SciTEWin::Replace() {
 	if (wFindReplace.Created()) {
 		if (replacing) {
 			SelectionIntoFind(false);
-			HWND hDlg = reinterpret_cast<HWND>(wFindReplace.GetID());
+			HWND hDlg = HwndOf(wFindReplace);
 			Dialog dlg(hDlg);
 			dlg.SetItemTextU(IDFINDWHAT, findWhat);
 			::SetFocus(hDlg);
@@ -1353,16 +1350,12 @@ void SciTEWin::Replace() {
 	} else {
 		if (searchStrip.visible || findStrip.visible)
 			return;
-		int dialog_id = (!props.GetInt("find.replace.advanced") ? IDD_REPLACE : IDD_REPLACE_ADV);
 
 		replacing = true;
 		havefound = false;
 
-		wFindReplace = ::CreateDialogParamW(hInstance,
-											(LPCWSTR)MAKEINTRESOURCE(dialog_id),
-											MainHWND(),
-											reinterpret_cast<DLGPROC>(ReplaceDlg),
-											reinterpret_cast<sptr_t>(this));
+		const int dialogID = (!props.GetInt("find.replace.advanced") ? IDD_REPLACE : IDD_REPLACE_ADV);
+		wFindReplace = CreateParameterisedDialog(MAKEINTRESOURCE(dialogID), ReplaceDlg);
 		wFindReplace.Show();
 	}
 }
@@ -1373,7 +1366,7 @@ void SciTEWin::FindReplace(bool replace) {
 
 void SciTEWin::DestroyFindReplace() {
 	if (wFindReplace.Created()) {
-		::EndDialog(reinterpret_cast<HWND>(wFindReplace.GetID()), IDCANCEL);
+		::EndDialog(HwndOf(wFindReplace), IDCANCEL);
 		wFindReplace.Destroy();
 	}
 }
@@ -1442,13 +1435,12 @@ BOOL SciTEWin::GoLineMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::GoLineDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::GoLineDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->GoLineMessage(hDlg, message, wParam);
 }
 
 void SciTEWin::GoLineDialog() {
-	DoDialog(hInstance, TEXT("GoLine"), MainHWND(), reinterpret_cast<DLGPROC>(GoLineDlg));
-	WindowSetFocus(wEditor);
+	DoDialog(TEXT("GoLine"), GoLineDlg);
 }
 
 BOOL SciTEWin::AbbrevMessage(HWND hDlg, UINT message, WPARAM wParam) {
@@ -1479,15 +1471,12 @@ BOOL SciTEWin::AbbrevMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::AbbrevDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::AbbrevDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->AbbrevMessage(hDlg, message, wParam);
 }
 
 bool SciTEWin::AbbrevDialog() {
-	bool success = (DoDialog(hInstance, TEXT("InsAbbrev"), MainHWND(),
-		reinterpret_cast<DLGPROC>(AbbrevDlg)) == IDOK);
-	WindowSetFocus(wEditor);
-	return success;
+	return DoDialog(TEXT("InsAbbrev"), AbbrevDlg) == IDOK;
 }
 
 BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
@@ -1544,13 +1533,12 @@ BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::TabSizeDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::TabSizeDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->TabSizeMessage(hDlg, message, wParam);
 }
 
 void SciTEWin::TabSizeDialog() {
-	DoDialog(hInstance, TEXT("TabSize"), MainHWND(), reinterpret_cast<DLGPROC>(TabSizeDlg));
-	WindowSetFocus(wEditor);
+	DoDialog(TEXT("TabSize"), TabSizeDlg);
 }
 
 bool SciTEWin::ParametersOpen() {
@@ -1559,7 +1547,7 @@ bool SciTEWin::ParametersOpen() {
 
 void SciTEWin::ParamGrab() {
 	if (wParameters.Created()) {
-		HWND hDlg = reinterpret_cast<HWND>(wParameters.GetID());
+		HWND hDlg = HwndOf(wParameters);
 		Dialog dlg(hDlg);
 		for (int param = 0; param < maxParam; param++) {
 			std::string paramVal = GUI::UTF8FromString(dlg.ItemTextG(IDPARAMSTART + param));
@@ -1614,7 +1602,7 @@ BOOL SciTEWin::ParametersMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::ParametersDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::ParametersDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->ParametersMessage(hDlg, message, wParam);
 }
 
@@ -1629,18 +1617,10 @@ bool SciTEWin::ParametersDialog(bool modal) {
 	bool success = false;
 	modalParameters = modal;
 	if (modal) {
-		success = DoDialog(hInstance,
-		                   TEXT("PARAMETERS"),
-		                   MainHWND(),
-		                   reinterpret_cast<DLGPROC>(ParametersDlg)) == IDOK;
+		success = DoDialog(TEXT("PARAMETERS"), ParametersDlg) == IDOK;
 		wParameters = 0;
-		WindowSetFocus(wEditor);
 	} else {
-		::CreateDialogParam(hInstance,
-		                    TEXT("PARAMETERSNONMODAL"),
-		                    MainHWND(),
-		                    reinterpret_cast<DLGPROC>(ParametersDlg),
-		                    reinterpret_cast<LPARAM>(this));
+		CreateParameterisedDialog(TEXT("PARAMETERSNONMODAL"), ParametersDlg);
 		wParameters.Show();
 	}
 
@@ -1649,7 +1629,7 @@ bool SciTEWin::ParametersDialog(bool modal) {
 
 SciTEBase::MessageBoxChoice SciTEWin::WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, MessageBoxStyle style) {
 	dialogsOnScreen++;
-	int ret = ::MessageBoxW(reinterpret_cast<HWND>(w.GetID()), msg.c_str(), appName, style | MB_SETFOREGROUND);
+	int ret = ::MessageBoxW(HwndOf(w), msg.c_str(), appName, style | MB_SETFOREGROUND);
 	dialogsOnScreen--;
 	switch (ret) {
 	case IDOK:
@@ -1718,13 +1698,11 @@ BOOL SciTEWin::AboutMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	return FALSE;
 }
 
-BOOL CALLBACK SciTEWin::AboutDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK SciTEWin::AboutDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
 	return Caller(hDlg, message, lParam)->AboutMessage(hDlg, message, wParam);
 }
 
 void SciTEWin::AboutDialogWithBuild(int staticBuild_) {
 	staticBuild = staticBuild_;
-	DoDialog(hInstance, TEXT("About"), MainHWND(),
-	         reinterpret_cast<DLGPROC>(AboutDlg));
-	WindowSetFocus(wEditor);
+	DoDialog(TEXT("About"), AboutDlg);
 }
